/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA – www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//---------------------------------------------------------------------------
// dbAdminController
//
// This controller interfaces with the dbadmin.html page to display
// status and provide actions to maintain the LSS Raw Data table.
// 
// For the ui-grid component view the following page:
// http://ui-grid.info/docs/#/api/ui.grid.class:GridOptions
//---------------------------------------------------------------------------
app.controller('dbAdminController', function ($scope, $rootScope, dbAdminService) {

    var epochDay = 86400;
    //-----------------------------------------------------------------------
    //                        LSS-100 Raw Data 
    //-----------------------------------------------------------------------
    $scope.dataStatus = [];
    // Configure the ui-grid
    $scope.lssDataStatusGridOptions = {
        enableRowSelection: false,
        enableSelectAll: false,
        multiSelect: false,
        minRowsToShow:1,
        enableColumnMenus: false,
        enableHorizontalScrollbar:0,
        enableVerticalScrollbar:0,
        pagingPageSizes:1,
        pagingPageSize: 1,
        columnDefs: [{field:'count', displayName:'Entries', width: '20%'}, 
                     {field:'oldest', displayName:'Oldest', width: '*'},   
                     {field:'newest', displayName:'Newest', width: '*'}]
    };
    // Set the data source.
    $scope.lssDataStatusGridOptions.data = $scope.dataStatus;
    // LSS-100 Reported data
    $scope.lssData = []; 
    $scope.lssReportDataGridOptions = {
        enableRowSelection: false,
        enableSelectAll: false,
        multiSelect: false,
        minRowsToShow:10,
        enableColumnMenus: false,
        enableHorizontalScrollbar:0,
        enableVerticalScrollbar:1,
        pagingPageSizes:1,
        pagingPageSize: 1,
        columnDefs: [{field:'id', displayName:'Data ID', width: '20%'},
                     {field:'OrgKey', displayName:'Organization', width: '*'}, 
                     {field:'TheaterNumber', displayName:'Theater', width: '20%'},   
                     {field:'DateTime', displayName:'LSS Date-Time', width: '*'}]
        
    };
    $scope.lssReportDataGridOptions.data = $scope.lssData;
 
    //-----------------------------------------------------------------------
    //                       Notifications Data 
    //-----------------------------------------------------------------------
    $scope.notificationsStatus = []; 
    // Configure the notifications UI grid.
    $scope.notificationStatusDataGridOptions = {
        enableRowSelection: false,
        enableSelectAll: false,
        multiSelect: false,
        minRowsToShow:1,
        enableColumnMenus: false,
        enableHorizontalScrollbar:0,
        enableVerticalScrollbar:0,
        pagingPageSizes:1,
        pagingPageSize: 1,
        columnDefs: [{field:'count', displayName:'Entries', width: '20%'},
                     {field:'oldest', displayName:'Oldest', width: '*'},
                     {field:'newest', displayName:'Newest', width: '*'}]
    };
    $scope.notificationStatusDataGridOptions.data = $scope.notificationsStatus;
    // Notifications data grid.
    $scope.notificationsList = [];
    $scope.notificationsDataGridOptions = {
        enableRowSelection: false,
        enableSelectAll: false,
        multiSelect: false,
        minRowsToShow:10,
        enableColumnMenus: false,
        enableHorizontalScrollbar:0,
        enableVerticalScrollbar:1,
        pagingPageSizes:1,
        pagingPageSize: 1,
        columnDefs: [{field:'RawDataId', displayName:'Data ID', width: '20%'},
                     {field:'ReceivedDateTime', displayName:'Data Received', width: '*'},
                     {field:'SentDateTime', displayName:'Send Status', width: '*'}]        
    };
    $scope.notificationsDataGridOptions.data = $scope.notificationsList;
    
    //-----------------------------------------------------------------------
    //                   LSS Report Data Delete Controls
    //-----------------------------------------------------------------------   
    // Public data properties (binding).
    $scope.lssDatePickerDisabled = true;
    $scope.lssDeleteDisabled = true;
    $scope.openedLssDatePicker = [false,false]; 
    $scope.cid = $rootScope.CID == EnumCID.cid ? "Authenticated":".";    
    $scope.datePickFormat = 'yyyy-MM-dd';
    $scope.lssDeleteStartDate;
    $scope.lssDeleteEndDate;   
    $scope.lssDatabaseCmdResult = new ValueItemStruct();
    // Private data
    var lssOldestEpoch = 0;
    var lssNewestEpoch = 0;
    var lssDeleteStartEpoch = 0;
    var lssDeleteEndEpoch= 0;

    //-----------------------------------------------------------------------
    // watchCollection dataStatus 
    //
    // When an update to LSS RawData status request response, setup the
    // date picker's start and end dates.
    //-----------------------------------------------------------------------
    $scope.$watchCollection('dataStatus', function(newValue){
        if(newValue.length > 0)
        {
            // We only want the year
            var parts = newValue[0].oldest.split(' ');
            $scope.lssDeleteStartDate = parts[0];
            $scope.lssDeleteEndDate = parts[0]; 
            $scope.lssDeleteDisabled = true;
            console.log('dbAdminController: Update the LSS Delete start and end dates');
        }
    });   

    //-----------------------------------------------------------------------
    // watchCollection lssData
    // 
    // When the LSS Raw Data (report data) is received, the date pickers
    // are enabled.
    //-----------------------------------------------------------------------
    $scope.$watchCollection('lssData', function(newValue){
        if(newValue.length > 0)
        {
            $scope.lssDatePickerDisabled = false;
        }            
    });

    //-----------------------------------------------------------------------
    // watch
    //
    // This watcher has the objectEquality set that will then enforce checking
    // of an objects properties (each property). When RawData records are 
    // deleted the result object is updated and is being watched. This update
    // will then execute a request to update both the status and RawData grid's
    // contents.
    //-----------------------------------------------------------------------
    $scope.$watch('lssDatabaseCmdResult', function(newValue){
        if(newValue.Boolean === true)
        {
            $scope.lssData = [];
            $scope.dataStatus = [];
            $scope.lssDataStatusGridOptions.data = $scope.dataStatus;
            $scope.lssReportDataGridOptions.data = $scope.lssData;
            dbAdminService.dbAdmin($scope.dataStatus, EnumDBAdminCmds.RAWDATA_STATISTICS,"","");
            dbAdminService.dbAdmin($scope.lssData, EnumDBAdminCmds.RAWDATA,"","");
        }
    },true)

    //-----------------------------------------------------------------------
    // openLssDatePicker
    //
    // Display the calendar for user date selection with the calendar 
    // glyphicon/button is activated.
    //-----------------------------------------------------------------------
    $scope.openLssDatePicker = function($event, $index) {
        $event.preventDefault();
        $event.stopPropagation();

        $scope.openedLssDatePicker[$index] = true;
    };
    
    //-----------------------------------------------------------------------
    // checkLssDateRange
    //
    // When a change to the input text is detected via a date picker 
    // selection. The dates are in units of epoch seconds that are then
    // ranged checked. When a valid date range has been selected, the 
    // delete button is then enabled.
    //-----------------------------------------------------------------------
    $scope.checkLssDateRange = function(){
        // The current date range in the database
        lssOldestEpoch = this.StringToEpochDate($scope.dataStatus[0].oldest);
        lssNewestEpoch = this.StringToEpochDate($scope.dataStatus[0].newest);
        // The current date picker range.
        lssDeleteStartEpoch = this.DateToEpoch($scope.lssDeleteStartDate);
        lssDeleteEndEpoch = this.DateToEpoch($scope.lssDeleteEndDate);
        // Range check to dates
        if(lssDeleteStartEpoch <= lssDeleteEndEpoch && lssDeleteEndEpoch <= lssNewestEpoch){
            $scope.lssDeleteDisabled = false;
        }else{
            $scope.lssDeleteDisabled = true;
        };        
    };
    
    //-----------------------------------------------------------------------
    // lssDeleteReportData
    //
    // Using the selected date range in epoch seconds, the date range is up
    // dated to include the previous and day after to ensure the date to be 
    // delected is within a day.
    //-----------------------------------------------------------------------
    $scope.lssDeleteReportData = function(){
        // Range check to dates, just in case delete button was enabled.
        if(lssDeleteStartEpoch <= lssDeleteEndEpoch && lssDeleteEndEpoch <= lssNewestEpoch){
            $scope.lssDatabaseCmdResult.Boolean = false;
            dbAdminService.dbAdmin($scope.lssDatabaseCmdResult,
                                     EnumDBAdminCmds.RAWDATA_DELETE, 
                                     lssDeleteStartEpoch - epochDay, 
                                     lssDeleteEndEpoch + epochDay);
        }else{
            $scope.lssDeleteDisabled = true;
        }           
    }
    
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    //-----------------------------------------------------------------------
    //                 Notifications Delete Controls
    //-----------------------------------------------------------------------
    //+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
    // public data (binding)
    $scope.notificationDatePickerDisable = true;
    $scope.notificationsDeleteDisabled = true;
    $scope.openedNotificationDatePicker = [false,false]; 
    $scope.notificationsDeleteStartDate;
    $scope.notificationsDeleteEndDate;
 
    $scope.notificationDatabaseCmdResult = new ValueItemStruct();
    // Private data
    var notificationOldestEpoch = 0;
    var notificationNewestEpoch = 0;
    var notificationDeleteStartEpoch = 0;
    var notificationDeleteEndEpoch= 0;
    

    //-----------------------------------------------------------------------
    // watchCollection notification status 
    //
    // When an update to LSS notifications status request response, setup the
    // date picker's start and end dates.
    //-----------------------------------------------------------------------
    $scope.$watchCollection('notificationsStatus', function(newValue){
        if(newValue.length > 0)
        {
            // We only want the year
            var parts = newValue[0].oldest.split(' ');
            $scope.notificationsDeleteStartDate = parts[0];
            $scope.notificationsDeleteEndDate = parts[0];
            $scope.notificationsDeleteDisabled = true;
        }
    }); 
    
    //-----------------------------------------------------------------------
    // watchCollection
    //
    // Enable the data picker when data is available.
    //-----------------------------------------------------------------------
    $scope.$watchCollection('notificationsList', function(newValue){
        if(newValue.length > 0)
        {
            $scope.notificationDatePickerDisable = false;
        }
    });
    
    //-----------------------------------------------------------------------
    // watch
    //
    // This watcher has the objectEquality set that will then enforce checking
    // of an objects properties (each property). When Notification records are 
    // deleted the result object is updated and is being watched. This update
    // will then execute a request to update both the status and RawData grid's
    // contents.
    //-----------------------------------------------------------------------
    $scope.$watch('notificationDatabaseCmdResult', function(newValue){
        if(newValue.Boolean === true)
        {
            $scope.notificationsStatus = [];
            $scope.notificationsList = [];
            $scope.notificationStatusDataGridOptions.data = $scope.notificationsStatus;
            $scope.notificationsDataGridOptions.data = $scope.notificationsList;
            dbAdminService.dbAdmin($scope.notificationsStatus, EnumDBAdminCmds.NOTIFICATION_STATISTICS,"","");
            dbAdminService.dbAdmin($scope.notificationsList, EnumDBAdminCmds.NOTIFICATIONS,"","");
        }
    },true)
    
    //-----------------------------------------------------------------------
    // openLssDatePicker
    //
    // Display the calendar for user date selection with the calendar 
    // glyphicon/button is activated.
    //-----------------------------------------------------------------------
    $scope.openNotificationsDatePicker = function($event, $index) {
      $event.preventDefault();
      $event.stopPropagation();

      $scope.openedNotificationDatePicker[$index] = true;
    };

    //-----------------------------------------------------------------------
    // checkNotificationDateRange
    //
    // When a change to the input text is detected via a date picker 
    // selection. The dates are in units of epoch seconds that are then
    // ranged checked. When a valid date range has been selected, the 
    // delete button is then enabled.
    //-----------------------------------------------------------------------
    $scope.checkNotificationDateRange = function(){
        // The current date range in the database
        notificationOldestEpoch = this.StringToEpochDate($scope.notificationsStatus[0].oldest);
        notificationNewestEpoch = this.StringToEpochDate($scope.notificationsStatus[0].newest);
        // The current date picker range.
        notificationDeleteStartEpoch = this.DateToEpoch($scope.notificationsDeleteStartDate);
        notificationDeleteEndEpoch = this.DateToEpoch($scope.notificationsDeleteEndDate);
        // Range check to dates
        if(notificationDeleteStartEpoch <= notificationDeleteEndEpoch && 
           notificationDeleteEndEpoch <= notificationNewestEpoch){
            $scope.notificationsDeleteDisabled = false;
        }else{
            $scope.notificationsDeleteDisabled = true;
        }
    }

    //-----------------------------------------------------------------------
    // deleteNotificationData
    //
    // Using the selected date range in epoch seconds, the date range is up
    // dated to include the previous and day after to ensure the date to be 
    // delected is within a day.
    //-----------------------------------------------------------------------
    $scope.deleteNotificationData = function(){        
        // Range check to dates, just in case delete button was enabled.
        if(notificationDeleteStartEpoch <= notificationDeleteEndEpoch &&
           notificationDeleteEndEpoch <= notificationNewestEpoch){
            $scope.notificationDatabaseCmdResult.Boolean = false;
            dbAdminService.dbAdmin($scope.notificationDatabaseCmdResult,
                                     EnumDBAdminCmds.NOTIFICATIONS_DELETE, 
                                     notificationDeleteStartEpoch - epochDay, 
                                     notificationDeleteEndEpoch + epochDay);
        }else{
            $scope.notificationsDeleteDisabled = true;
        }
    }

    //-----------------------------------------------------------------------
    //                         Common Functions
    //-----------------------------------------------------------------------
    //-----------------------------------------------------------------------
    // DateToEpoch
    //
    // Converts a year, month 0 - 11, and day of the month to epoch seconds.
    // Minutes, Seconds, and milliseconds are ignored.
    //-----------------------------------------------------------------------
    $scope.DateToEpoch = function(value)
    {
        var epochValue = 0;
        
        if(value instanceof Date)
        {
            epochValue = Date.UTC(value.getFullYear(), value.getMonth(), value.getDate())/1000;            
        }
        else
        {
            var pieces = value.split('-');
            var year  = parseInt(pieces[0]);
            var month = parseInt(pieces[1])-1;
            var day   = parseInt(pieces[2]);
            epochValue = Date.UTC(year, month, day)/1000;
        }
        
        if(epochValue < 0)
            return 0;
        else
            return epochValue;
    };
    
    //-----------------------------------------------------------------------
    // StringToEpochDate
    //
    // Takes a date string '1920-01-01' to epoch seconds.
    //-----------------------------------------------------------------------
    $scope.StringToEpochDate = function(value)
    {       
        var pieces = value.split('-');
        var year  = parseInt(pieces[0]);
        var month = parseInt(pieces[1])-1;
        var daytime = pieces[2].split(' ');
        var day = parseInt(daytime[0]);
        var epochValue = Date.UTC(year, month, day)/1000;
        if(epochValue < 0)
            return 0;
        else
            return epochValue;
    };



    //-----------------------------------------------------------------------
    //                          Web Services
    //-----------------------------------------------------------------------
    // Get the raw data status, LSS data measurements posted to the Web service.
    dbAdminService.dbAdmin($scope.dataStatus, EnumDBAdminCmds.RAWDATA_STATISTICS,"","");
    // Get the notification database status.
    dbAdminService.dbAdmin($scope.notificationsStatus, EnumDBAdminCmds.NOTIFICATION_STATISTICS, "","");
    
    // Get all LSS reported data records
    dbAdminService.dbAdmin($scope.lssData, EnumDBAdminCmds.RAWDATA,"","");
    // Get all notifications
    dbAdminService.dbAdmin($scope.notificationsList, EnumDBAdminCmds.NOTIFICATIONS,"","");

});