/*******************************************************************************
* 
*    ULTRA STEREO LABS CONFIDENTIAL
*    _____________________________________________________________________
* 
*       Copyright (c) 2014 - 2015 Ultra Stereo Labs Incorporated 
*                         All Rights Reserved.
* 
*    NOTICE:  All information contained herein is, and remains
*    the property of Ultra Stereo Labs Incorporated and its suppliers,
*    if any.  The intellectual and technical concepts contained
*    herein are proprietary to Ultra Stereo Labs Incorporated
*    and its suppliers and may be covered by U.S. and Foreign Patents,
*    in process, and are protected by trade secret or copyright law.
*    Dissemination of this information or reproduction of this material
*    is strictly forbidden unless prior written permission is obtained
*    from Ultra Stereo Labs Incorporated. 181 Bonetti Drive, San Luis Obispo, 
*    California 93401-7397, USA – www.uslinc.com
*
*    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, 
*    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
*    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
*    IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
*    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
*    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
*    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*
*
*******************************************************************************/
//---------------------------------------------------------------------------
// lssDataDisplayController
//
// This controller request all LSS data items and then updateds bindings 
// via a watch on the collection. The controller will also request the groups
// the user has access to, and parse the data from the LSS into a grouped-
// hierarchical view.  The views, when expanded, will trigger detail ajax calls
// to the database for the related data.  Clicking an individual field will 
// call the database for the detailed information need to graph the data in the
// supplied date range.
//---------------------------------------------------------------------------
app.controller('lssDataDisplayController', function ($scope, $timeout, $location,
        allDataListSharedFactory, getParametersFactory, configurationFactory,  dbRequestService) {
    // Initialize our bindings
    $scope.settings = configurationFactory;
    $scope.getParameters = getParametersFactory;
    $scope.theaterlist = allDataListSharedFactory;  // Create a list for just Theaters
    $scope.currentPage = 1; //current page
    $scope.totalItems = 5;  //var for watcher function
    $scope.ootTheaters = [];  // Array of theaternumbers oot
    $scope.ootAuditoriums = [];  //Object of theaternumber:Auditoriumnumber for oot
    $scope.tab = 1;  // Graph pane display
    $scope.opened = [false,false];  // For datepicker display control
    // Get user groups
    $scope.theatergroups = [];
    $scope.groups = [];
    $scope.datataborder= 'x'; //Order data view by date
    $scope.dtrev = false;
    //Declaration for the Graph Display
    $scope.settings.chartSettings.chart['height']=450;  //Need to force to resize to device
    $scope.settings.chartSettings.title.enable = false;
    $scope.settings.chartSettings.chart['xAxis'] = {
        axisLabel: 'Date',
        tickFormat: function(d){ 
            return d3.time.format($scope.settings.ChartDateFormat)(new Date(d*1000)); 
        }
    };
    $scope.settings.chartSettings.chart['yAxis']= {
        axisLabel: 'Field Value',
        tickFormat: function(d){
            if(d===null){return "None";}
            return (TickToString(d)+" "+$scope.data[0].values[0].units);
        },
        axisLabelDistance: 6
    };
    $scope.data = [
                {values:[],key:"Descript",color:'#33F'},
                {values:[],key:'Maximum',color:'#f50'},
                {values:[],key:'Minimum',color:'#f00'}
            ];
    
    // Controller functions.
    // Request the data for populating the last report informationgetTime
    dbRequestService.dbRequest($scope.groups, EnumDbCmds.MY_GROUPS, "", "");
    dbRequestService.dbRequest($scope.theatergroups, EnumDbCmds.MY_THEATERGROUPS, "", "");
    // Attached a watcher on the list object, collection.
    $scope.$watchCollection('theaterlist', function(newNames, oldNames){
        console.log('Collection update length ='+newNames.length);
        if($scope.theatergroups.length > 0 ){    $scope.checkOOT(); }
    });
    $scope.$watchCollection('groups', function(newNames, oldNames){
        if($scope.groups.length===1){$scope.istopgroupopen = true;}
        if($scope.theatergroups.length > 0 ){    $scope.checkOOT(); }
    });
    $scope.$watchCollection('theatergroups', function(newNames, oldNames){
        if($scope.theaterlist.length > 0 ){    $scope.checkOOT(); }    
    });
    //Find the latest LSS record in the currently filtered set for each Description
    $scope.ootInProgress = false;
    $scope.checkOOT = function(){
        if($scope.ootInProgress){ return;}  // Override in case of many quick data calls
        $scope.ootInProgress = true;
        for (i in $scope.theatergroups){
            $scope.theatergroups[i]["oot"] = 0;
        }
        for (i in $scope.groups){
            $scope.groups[i]["oot"] = 0;
        }
        $scope.ootTheaters.length=0;
        $scope.ootAuditoriums.length=0;
        for (i in $scope.theaterlist) {
            if ($scope.theaterlist[i].OutsideLimits==="1"){
                var result = $.grep($scope.theatergroups, function(e){ return e.TheaterNumber===$scope.theaterlist[i].TheaterNumber && e.OrgKey===$scope.theaterlist[i].OrgKey; });
                for (i in result){
                    result[i]["oot"]+=1;
                }
            }
        }
        for (i in $scope.groups){
            var result = $.grep($scope.theatergroups, function(e){ return e.GroupId===$scope.groups[i].GID && e.oot>0; });
            $scope.ootPIDs($scope.groups[i], result.length);
        }
        $scope.ootInProgress = false;
    };
    // Apply the OOT flag to all the Parent Groups
    $scope.ootPIDs = function(g,oot){
        g.oot += oot;
        var result = $.grep($scope.groups, function(e){ return e.GID===g.PID; });
        for(var i=0; i<result.length; i++){
            $scope.ootPIDs(result[i], oot);
        }
    };
    //Find the name of the theater from a theatergroup;
    $scope.TheaterDisplayName = function(tg){
        var i=0;
        var res=0;
        while(i<$scope.theaterlist.length){
            elem=$scope.theaterlist[i];
            if(elem.TheaterNumber === tg.TheaterNumber && elem.OrgKey === tg.OrgKey){
                if(elem.DateTime > $scope.theaterlist[res].DateTime || res===0){ res=i; }
            }
            i++;
        }
        // Check for an empty list
        if($scope.theaterlist.length > 0)
        {        
            result = $scope.theaterlist[res].TheaterName;
            if(res===0){ if($scope.theaterlist[0].OrgKey!==tg.OrgKey || $scope.theaterlist[0].TheaterNumber!==tg.TheaterNumber){
                result = "No Reports Found";
            }}
            return result;
        }else{
            return "";
        }
    };
    $scope.setPage = function(pageNo) {
        $scope.currentPage = pageNo;
    };
    $scope.filter = function() {
        $timeout(function() { 
            $scope.filteredItems = $scope.filtered.length;
        }, 10);
//        $scope.parseLatest();
    };
    //sort function for LSS Table headers
    $scope.predicate = "DateTime";
    $scope.reverse = false;
    $scope.sort_by = function(predicate) {
        if ($scope.predicate === predicate){ $scope.reverse = !$scope.reverse;}
        else { $scope.reverse = false; }
        $scope.predicate = predicate; 
    };
    // Click handler for LSS data table (show graphs).
    $scope.LSSGraph = function ($LSS){
        //Show the container that has the graph in it
         $("#LSSGraphModal").modal('show');
         $scope.data = [
                {values:[],key:"Descript",color:'#33F'},
                {values:[],key:'Maximum',color:'#f50'},
                {values:[],key:'Minimum',color:'#f00'}
            ];
        //Get the data to graph into the graphing data collection
        
        //Change the title, tooltips and other pieces to match
        $scope.graphtitle="Theater "+$LSS.TheaterNumber+" Screen "+$LSS.AuditoriumNumber+" - "+$LSS.Description;
        $scope.data[0].key = $LSS.Description;
        $scope.settings.chartSettings.chart.yAxis.axisLabel = $LSS.Description;
        $scope.tab = 1;
        // Delay getting the data until the animation to show the modal has finished (500ms). 
        // This prevents the NVD3 graph from rendering "smashed" (no width).
        setTimeout(function(){ 
            dbRequestService.dbGraphRequest($scope.data, EnumDbCmds.GRAPH_DATA, $scope.getParameters.startdate.getTime()/1000, $scope.getParameters.enddate.getTime()/1000, $LSS.TheaterNumber, $LSS.AuditoriumNumber, $LSS.Description, $LSS.OrgKey);  
        },300);
    };
    //Date Selection
    // Open the date picker
    $scope.open = function($event, $index) {
      $event.preventDefault();
      $event.stopPropagation();
      $scope.opened[$index] = true;
    };
    // Re-aquire the view data when the date range changes
    $scope.checkDateRange = function(){
        console.log('Date Range Change; updating records.');
        dbRequestService.dbRequest($scope.theaterlist, EnumDbCmds.THEATER_OVERVIEW, $scope.getParameters.startdate.getTime()/1000, $scope.getParameters.enddate.getTime()/1000);
    };
    $scope.checkDateRange();
});