#!/usr/bin/python
'''
Performs all MySQL operations on the LSSDB
Created on Sep 11, 2015

@author: MTA
'''

import GlobalVars as gv
import MySQLdb
import getpass
import os, sys
import tarfile
from string import upper

Verbose = True  # Set tp true to print statements as progress continues
# dbName = "lssdb"  # the name of the database to be added or modified
# dbSetupFile = "DB_SETUP.sql"
# newDBfile = "lssdb_default_150715.sql"
# setuptarfile = "LssDatabase_v1_0_150804.tar.gz"

# Compare table column names and properties
# If name does not exist create a new one, old will be deleted at end.
def compareColumns( columnSQL, dbColumn ):
    # Check each column for a match
    for column in columnSQL:
        print column
    return

# Get the released tarball and extract the DB_SETUP.SQL file for use in comparing
# any current database or building a virgin one from scratch.
def retrieveTarFiles( tarFolder, tarcollectionname, tarfilename ):
    # For now assume the tarball is in the current folder same as this program.
    # Change to current working dir because pathnames give open a problem
    print tarFolder
    olddir = os.getcwd()
    try:
        os.chdir( tarFolder )
    except:
        print "Remaining in current folder: %s" % olddir
    tar = tarfile.open( tarcollectionname, "r:gz" )
    collection = tar.getmembers()
    for member in collection:
        if tarfilename in member.name:
            print "SQL Command file: %s found." % ( member.name )
            try:
                os.chdir( olddir )
            except:
                print "Unable to revert to original folder."
            return tar.extractfile( member ).read()
    return "File not Found"

# Execute the SQL string contained in the open file string
# called sqlsource.  This sub should be used only for
# commands requiring a database commit function.
def executeSQLStatement( sstring, sqlsource ):
    global cursor, db
    ln = len( sstring )
    ptr1 = 0
    print "Checking for changes to database."
    while True:
        ptr3 = sqlsource[ptr1:].find( sstring )
        if ptr3 < 0:
            break
        ptr3 = ptr3 + ptr1  # Offset to absolute start of statement
        ptr1 = ptr3 + ln  # offset to db name accounting for length of search field
        ptr2 = sqlsource[ptr1:].find( "(" )
        # We have to trim the table name because not every sql file uses the same delimiters
        # some are 'name' and some are "name" and some are name and some are `name`
        tableName = sqlsource[ptr1:ptr1 + ptr2].replace( "'", "" ).replace( '"', "" ).replace( "`", "" ).strip()
        ptr1 = sqlsource[ptr3:].find( ";" )
        sqlstatement = sqlsource[ptr3:ptr1 + ptr3 + 1]
        ptr1 = ptr3 + ptr1 + 1
        print "Table: %s" % tableName
        try:
            if Verbose: print sqlstatement
            cursor.execute( sqlstatement )
            db.commit()
            results = cursor.fetchall()
            if Verbose: print results
        except:
            print "Error in SQL Operation. Status Unknown"
            return -1
    return 0

def lprint( text, printtoo = True ):
    if( printtoo ):
        print( text )
    try:
        gv.LogFileHandle.write( text + "\n" )
    except:
        return

def DBoperations():
    global gv
    try:
        os.chdir( gv.WorkingDir )
    except:
        print "Using folder: %s" % os.getcwd()
    """
    First we check to see if we can access the database as root. without using a
    password. If we can, we go ahead and do things, if not we ask the user for
    the administrative account and password.
    """
    # Open database connection
    defaultuser = False
    try:
        db = MySQLdb.connect( "localhost", "root", "" )
    except:
        # Try the second favorite login to root
        try:
            db = MySQLdb.connect( "localhost", "root", "admin" )
        except:
            # If we can't get in as root, lets see if the database is already set up with lssdb
            try:
                db = MySQLdb.connect( "localhost", "lssdb", "lssdb" )
                defaultuser = True
            except:
                tryagain = True
                print "The default database credentials do not work!"
                print "Please enter the administrative username and password."
                while tryagain:
                    username = raw_input( "Please enter the User Name: " )
                    pw = getpass.getpass( "Please enter the password: " )
                    try:
                        db = MySQLdb.connect( "localhost", username, pw )
                        tryagain = False
                    except:
                        print "The attempt to open the database failed!"
                        result = raw_input( "Try again? [Y/n]" )
                        if len( result ) > 0:
                            if upper( result[0] ) == "N" :
                                print "Goodbye"
                                lprint( "Database access could not be made with current username and password.", False )
                                sys.exit( 1 )

    """
    Then we check for the existence of an LSSDB db and if there is none then
    we assume a virgin db and set one up.  Otherwise if one exists we assume
    this is a program update and add new portions of the database while trying
    to preserve the existing LSSDB.
    """
    # prepare a cursor object using cursor() method
    cursor = db.cursor()
    # Get a list of current databases
    cursor.execute( "SHOW DATABASES" )
    results = cursor.fetchall()
    dbexists = False
    for item in results:
        if item[0] == gv.dbName:
            dbexists = True
            lprint( "Database %s is currently installed." % gv.dbName )
    if dbexists:
        setupfile = retrieveTarFiles( gv.WorkingDir, gv.setuptarfile, gv.dbSetupFile )
        # We have an existing database.  Check it for completeness
        # Need to traverse thru sql file line by line
        ptr1 = setupfile.find( "USE" ) + 3
        ptr2 = setupfile[ptr1:].find( ";" )
        gv.dbName = setupfile[ptr1:ptr1 + ptr2].strip()
        try:
            # Check and see if db is usable at this level
            cursor.execute( "USE " + gv.dbName )
            print "Database %s now being used." % gv.dbName
        except:
            # if not make one
            cursor.execute( "CREATE DATABASE IF NOT EXISTS " + gv.dbName )
            cursor.execute( "USE " + gv.dbName )
            print "Database %s now being created." % gv.dbName

        # Now get a list of tables specified in the sql setup
        # We could have used a function for SQL commands but during the
        # table operations we want to fix any columns that have changed.
        sstring = r"CREATE TABLE IF NOT EXISTS"
        ln = len( sstring )
        ptr1 = 0
        print "Checking for changes to database."
        while True:
            """ NOTE: SQL Table commands have the following structure:
            CREATE TABLE IF NOT EXISTS `users` (
              `UID` int(11) NOT NULL AUTO_INCREMENT,
              `UserName` varchar(50) NOT NULL,
              `Password` varchar(200) NOT NULL,
              `Company` varchar(50) NOT NULL,
              `Level` int(2) NOT NULL DEFAULT 0,
              `NotificationType` int(2) NOT NULL,
              `Email` varchar(50) NOT NULL,
              `Phone` varchar(100) NOT NULL,
              `Created` timestamp NOT NULL DEFAULT CURRENT_TIMESTAMP,
              PRIMARY KEY (`UID`)) ENGINE=InnoDB  DEFAULT CHARSET=latin1 AUTO_INCREMENT=1 ;
            """
            ptr3 = setupfile[ptr1:].find( sstring )
            if ptr3 < 0:
                break
            ptr3 = ptr3 + ptr1  # Offset to absolute start of statement
            ptr1 = ptr3 + ln  # offset to db name accounting for length of search field
            ptr2 = setupfile[ptr1:].find( "(" )
            # We have to trim the table name because not every sql file uses the same delimiters
            # some are 'name' and some are "name" and some are name and some are `name`
            tableName = setupfile[ptr1:ptr1 + ptr2].replace( "'", "" ).replace( '"', "" ).replace( "`", "" ).strip()
            ptr1 = setupfile[ptr3:].find( ";" )  # Find the end of the statement
            sqlstatement = setupfile[ptr3:ptr1 + ptr3 + 1]
            ptr1 = ptr3 + ptr1 + 1
            # Now get each SQL statement in the SQL command
            sqlstructure = []
            xptr3 = 0
            while True:
                xptr1 = sqlstatement[xptr3:].find( "\n" )
                if xptr1 < 0: break
                xptr1 = xptr3 + xptr1 + 1
                xptr2 = sqlstatement[xptr1:].find( "," )
                if xptr2 < 0: break
                sqlstructure.append( sqlstatement[xptr1:xptr1 + xptr2].split() )
                xptr3 = xptr2 + xptr1
            print "Table: %s" % tableName
            try:
                cursor.execute( "SHOW COLUMNS IN " + tableName )
                columns = cursor.fetchall()
                if Verbose:
                    for item in columns:
                        print "Column: %s" % item[0], item[1]
                # check for changes
                for item in sqlstructure:
                    tableOK = True
                    for checkitem in columns:
                        if item[0].replace( "`", "" ).replace( "'", "" ).replace( '"', "" ) == checkitem[0]:
                            if item[1] == checkitem[1]:
                                print "Column Match - column: %s, type; %s " % ( checkitem[0], checkitem[1] )
                                break
                            else:
                                tableOK = False
                if tableOK:
                    print "All columns are in proper format!"
                else:
                    print "There are errors in one or more tables."
            except:
                print "No Table %s found.  Now creating one." % tableName
                if Verbose: print sqlstatement
                cursor.execute( sqlstatement )
                columns = cursor.fetchall()
                """ Insert commands are only required for new builds of a "virgin" database
                    for now we load in a save default database and do not have a virgin version.
                    # The database is OK, now execute the INSERT statements"""
                cursor.execute( "USE " + gv.dbName )
                sstring = r"INSERT INTO"
                result = executeSQLStatement( sstring, setupfile )

    else:
        # No database exists so start adding as if virgin
        lprint( "No %s database exists. Installing initial database." % gv.dbName )
        cursor.execute( "CREATE DATABASE IF NOT EXISTS " + gv.dbName )
        cursor.fetchall()
        cursor.execute( "USE " + gv.dbName )
        lprint( "Database %s now being created." % gv.dbName )
        # print "Debug - WorkingDir: %s  setuptarfile: %s  newDBfile: %s" % ( gv.WorkingDir, gv.setuptarfile, gv.newDBfile )
        defaultdb = str( retrieveTarFiles( gv.WorkingDir, gv.setuptarfile, gv.newDBfile ) )
        lprint( "Loading initial database with file: %s" % gv.newDBfile )
        cursor.execute( defaultdb )
        result = cursor.fetchall()
        if len( result ) == 0:
            lprint( "The database has been successfully loaded." )
        else:
            lprint( "There was an error installing the database." )
    cursor.close()

    ##########################################################
    #                                                        #
    #       Now create the lssdb user account                #
    #                                                        #
    ##########################################################

    print "Checking for existing 'lssdb' user account."
    if not defaultuser:
        cursor = db.cursor()
        cursor.execute( "USE mysql" )
        result = cursor.fetchall()
        cursor.execute( "SELECT EXISTS(SELECT 1 FROM user WHERE user = 'lssdb');" )
        result = cursor.fetchall()
        if result[0][0] == 1:
            lprint( "User 'lssdb' already exists in database." )
        else:
            lprint( "Creating 'lssdb' user with all privileges on lssdb." )
            try:
                db.query( "CREATE USER 'lssdb'@'localhost' IDENTIFIED BY 'lssdb';" )
                # Have to grant *.* because no way to log into a specific database
                db.query( "GRANT ALL PRIVILEGES ON *.* TO lssdb@localhost WITH GRANT OPTION" )
                lprint( "User 'lssdb' was successfully created." )
            except:
                lprint( "ERROR: Unable to create user 'lssdb'." )
    else:
        lprint( "User 'lssdb' already exists in database." )
    cursor.close()
    db.close()

def main():
    gv.init()
    print "Performing DBoperations in stand alone mode."
    try:
        gv.LogFileHandle = open( "lssdb_log.txt", "w" )
    except:
        print "Unable to open log file. Continuing without."
    try:
        DBoperations()
        gv.LogFileHandle.close()
        print "Program completed normally."
    except KeyboardInterrupt:
        gv.LogFileHandle.close()
        exit()

if __name__ == '__main__':
    try:
        main()
    except:
        raise
