#!/usr/bin/python
#
# Program to load LSSDB files for and create lssdb user
# while analyzing and logging current system configuration
#
# Last modified: 10-28-2015        By: M. Aarons
#
#####################################################
#                                                   #
#       GLOBALS and Imports                         #
#                                                   #
#####################################################

import os
import imp
import getpass
import GlobalVars as gv
import subprocess
import AnsiColor as ac
import platform
from datetime import datetime

def lprint( text, printtoo = True ):
    global LogFileHandle
    if( printtoo ):
        print( text )
    try:
        LogFileHandle.write( text + "\n" )
    except:
        return

def main():
    global gv
    gv.init()
    # Open a log file to save all data collected from segments
    try:
        gv.LogFileHandle = open( "lssdb_log.txt", "w" )
        gv.LogFileHandle.write( "USL LSSDB Setup check. Check Performed: " + datetime.now().strftime( "%Y-%m-%d  %I:%M:%S %p" ) )
    except:
        print "Unable to open log file. Continuing without."

    if platform.dist()[0] == "Ubuntu":
        ac.cprint( "It's best to run this program as root but if you have administrative privileges it will run as super user." )
        ac.cprint( "Please enter your administrative password: ", ac.YELLOW, False )
        gv.suPassword = getpass.getpass( "" )
    # Make sure the MySQLdb module exists. Fedora sometimes does not have it.
    try:
        imp.find_module( 'MySQLdb' )
    except ImportError:
        lprint( "The MySQLdb module for Python is missing and is required to run this program." )
        response = raw_input( "OK to install MySQLdb for Python? [Yn]" )
        if( len( response ) == 0 or response[0].upper() == "Y" ):
            # Load the module.  There are 5 ways to load but for now we will worry only about Fedora & Ubuntu
            # 1.easy_install mysql-python (mix os)
            # 2.pip install mysql-python (mix os)
            # 3.apt-get install python-mysqldb (Linux Ubuntu, ...)
            # 4.cd /usr/ports/databases/py-MySQLdb && make install clean (FreeBSD)
            # 5.yum install MySQL-python (Linux Fedora, CentOS ...)
            cmd = "echo %s | sudo -S apt-get -y install python-mysqldb" % gv.suPassword
            if platform.dist()[0] <> "Ubuntu":
                cmd = "yum -y install MySQL-python"
            p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
            output = p.stdout.read()
            lprint( output )
        else:
            lprint( "User abort.  Good-bye." )
            exit()
    try:
        print( "="*60 )
        gv.WorkingDir = os.getcwd()
        print "Starting Getinfo"
        import GetInfo
        GetInfo.GetInfo()
        print "Getinfo Done"
        print "Starting DBoperations"
        import DBoperations
        DBoperations.DBoperations()
        print "DBoperations Done"
        print "Starting File Operations"
        import FileOps
        FileOps.FileOps()
        print "File Operations Done"
        print "Starting ServerOps"
        import ServerOps
        ServerOps.ServerOps()
        print "ServerOps Done"
        print "Starting SecurityOps"
        import Security
        Security.SecurityOps()
        print "SecurityOps Done"
        print( "="*60 )
    except KeyboardInterrupt:
        exit()
        gv.LogFileHandle.close()
        ac.cprint( "Program terminated by user.", ac.RED )
        exit()
    # except:
    #    ac.cprint( "Program has ended with errors.", ac.RED )
    #    print( "="*60 )
    #    gv.LogFileHandle.close()
    #    exit()
    gv.LogFileHandle.close()
    print "Program completed normally."
    ac.cprint( "Results are located in lssdb_log.txt" , ac.YELLOW )

if __name__ == '__main__':
    try:
        main()
    except:
        raise
