#!/usr/bin/python
'''
Performs all PHP Server operations for the LSSDB install
Created on Sep 24, 2015

@author: MTA
'''

import AnsiColor as ac
import os
import subprocess
import GlobalVars as gv

def lprint( text, printtoo = True ):
    if( printtoo ):
        print( text )
    try:
        gv.LogFileHandle.write( text + "\n" )
    except:
        return

def findOverride( configFile, override ):
    # Look for a directory tag with the document root
    ptr1 = configFile.find( "<Directory %s" % gv.lssRoot )
    if ptr1 < 0:
        text = "No Document Root Directory tag was found in the configuration file."
        ac.cprint( text, ac.RED )
        lprint( text, False )
    else:
        text = "Document Root Directory tag was found in the configuration file."
        ac.cprint( text, ac.GREEN )
        lprint( text, False )
        ptr2 = configFile[ptr1:].find( "</Directory>" )
        dirTag = configFile[ptr1:ptr1 + ptr2 + 12]
        lprint( dirTag, False )
        ptr1 = dirTag.find( "AllowOverride" )
        if ptr1 < 0:
            lprint( "No AllowOverride directive found." )
        else:
            # Check to see if AllowOverride is All
            # Note that "All" is in AllowOverride so a little tougher
            ptr2 = dirTag[ptr1:].find( "\n" )
            ort = dirTag[ptr1:ptr1 + ptr2].lower()
            if ort.rfind( "all" ) < 13:
                text = "The AllowOverride directive must be set to All.\n It is currently: %s" % ort
                ac.cprint( text, ac.RED )
                lprint( text, False )
                ac.cprint( dirTag, ac.RED )
            else:
                text = "The AllowOverride directive is properly set to All."
                ac.cprint( text, ac.GREEN )
                lprint( text, False )
                ac.cprint( dirTag, ac.GREEN )
                override = True
    return override

def ServerOps():
    """ There are four ways to specify virtual hosts in Apache
    The first method is to specify it directly in the http.conf file and
    will look something like the following tag:
    
    <VirtualHost *:80>
        ServerName webtest190.com
        ServerAlias *.webtest190.com
        DocumentRoot /var/www/webtest
    </VirtualHost>
    
    The second method is to specify it in a Vhost file which will appear
    in the http.conf file as:
    
    Include conf/extra/httpd-vhosts.conf
    
    The third way is to include it in a folder pointed to in the http.conf
    file and appears like:
    
    # Include the virtual host configurations:
    Include sites-enabled/
    
    This method is the toughest to understand because the files are
    really in the sites-available folder and just linked to the site-enabled.
    Or if the user didn't understand the link concept, the files may directly
    be in the sites-available folder.
    
    The fourth method is to include the files in the standard conf.d folder that is usually
    included as default Apache set ups.  The directive looks like:
    
    # Load config files in the "/etc/httpd/conf.d" directory, if any.
    IncludeOptional conf.d/*.conf
    
    """
    global gv
    vhdfound = False
    # Check what kind of platform we are on
    if gv.sysPlatform == "Windows":
        print "Checking Windows Distribution for 'httpd.conf'"
        print "Looking for Apache folder."
        cmd = "dir c:\\http*.exe /B/S"
        p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
        output = p.stdout.read()
        if output.find( "Not Found" ) > 0:
            output = ""
            try:
                cmd = "dir d:\\http*.exe /B/S"
                p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                output = p.stdout.read()
                if output.find( "Not Found" ) > 0:
                    output = ""
            except:
                pass
        apache = output.split( "\n" )
        lprint( "Apache found at: %s" % apache[0] )
        # Assume that Apache in in some bin folder and path may not have apache in it
        apachedir = apache[0][0:apache[0].find( "\\bin" )]
        cmd = 'dir ' + apachedir + '\conf\httpd.conf /S /B'
        p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
        output = p.stdout.read()
        if output.find( "File Not Found" ) >= 0:
            print "The Apache Configuration file was not found."
        else:
            result = output.split( "\n" )
            # Can get two results, the stored original or the users current copy
            for item in result:
                if item.find( "original" ) < 0:
                    filename = item
                    break
            lprint( "Found: %s" % filename )
            path = filename[0:filename.rfind( "\\" ) + 1]
            # Isolate the root path for Apache
            rootpath = path[0:( path[:-1].rfind( "\\" ) ) + 1]
            lprint( "Path: %s  rootpath: %s" % ( path, rootpath ) )
            os.chdir( path )
            fh = open( "httpd.conf", 'r' )
            configFile = fh.read()
            fh.close()
            # print configFile
            vsearch = True
            ptr2 = 0
            while vsearch:
                ptr2 = configFile.find( "</VirtualHost>", ptr2 )
                if( ptr2 > 0 ):
                    # If Virtual host specification in config, find the path
                    ptr2 = ptr2 + 15
                    ptr1 = configFile.rfind( "<VirtualHost", 0, ptr2 )
                    vhosttag = configFile[ptr1:ptr2]
                    if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                        vsearch = False
                        lprint ( "Virtual Host Directive found." )
                        lprint ( vhosttag )
                        vhdfound = True
                        ptr1 = vhosttag.find( "DocumentRoot" )
                        if( ptr1 > 0 ):
                            ptr2 = vhosttag[ptr1:].find( "\n" )
                            gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].replace( '"', '' ).replace( "'", '' ).strip()
                else:
                    vsearch = False
            # At this point if lssRoot has length then we have a path
            # If not look for a directive using the -vhost.conf file
            if not vhdfound:
                # Still have not found the path for lssdb. Look for second method
                ptr2 = configFile.find( "-vhosts.conf" )
                if( ptr2 > 0 ):
                    ptr2 = ptr2 + 12
                    ptr1 = configFile.rfind( "Include", 0, ptr2 )
                    vhosttag = configFile[ptr1:ptr2]
                    lprint( "Virtual host definition found: %s" % vhosttag )
                    vhostfile = vhosttag.split( " " )[1]
                    os.chdir( apachedir )
                    fh = open( vhostfile, 'r' )
                    vhostfile = fh.read()
                    fh.close()
                    # print vhostfile
                    vsearch = True
                    ptr2 = 0
                    while vsearch:
                        ptr2 = vhostfile.find( "\n</VirtualHost>", ptr2 )
                        if( ptr2 > 0 ):
                            # If Virtual host specification in config, find the path
                            ptr2 = ptr2 + 15
                            ptr1 = vhostfile.rfind( "<VirtualHost", 0, ptr2 )
                            vhosttag = vhostfile[ptr1:ptr2]
                            if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                                vsearch = False
                                vhdfound = True
                                lprint ( "Virtual Host Directive found." )
                                lprint ( vhosttag )
                                ptr1 = vhosttag.find( "DocumentRoot" )
                                if( ptr1 > 0 ):
                                    ptr2 = vhosttag[ptr1:].find( "\n" )
                                    gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].replace( '"', '' ).replace( "'", '' ).strip()
                        else:
                            vsearch = False
            # At this point if lssRoot has length then we have a path
            # If not look for a directive using the sites-enabled folder file
            # This is the latest recommended way to do vhosts and the file names
            # are usually the names of the URL like mywebsite.com.  The files should
            # really be in the sites-available folder with links to the sites-enabled folder.
            # The directive in the config file usually looks like:
            #     #Include the virtual host configurations:
            #     Include sites-enabled/
            if not vhdfound:
                # Still have not found the path for lssdb. Look for third method
                ptr2 = configFile.find( "sites-enabled/" )
                if( ptr2 > 0 ):
                    ptr2 = ptr2 + 14
                    ptr1 = configFile.rfind( "Include", 0, ptr2 )
                    vhosttag = configFile[ptr1:ptr2]
                    lprint( "Virtual host definition found: %s" % vhosttag )
                    vhostfile = vhosttag.split( " " )[1]
                    os.chdir( apachedir + vhosttag )
                    cmd = 'dir /A:-D /B'
                    p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                    output = p.stdout.read()
                    filelist = output.split( "\n" )
                    for filename in filelist:
                        # Each file should contain something like the following:
                        # <VirtualHost *:80>
                        #    ServerName webtest190.com
                        #    ServerAlias *.webtest190.com
                        #    DocumentRoot /var/www/webtest
                        # </VirtualHost>
                        fh = open( filename, 'r' )
                        vhostfile = fh.read()
                        fh.close()
                        # print vhostfile
                        vsearch = True
                        ptr2 = 0
                        while vsearch:
                            ptr2 = vhostfile.find( "\n</VirtualHost>", ptr2 )
                            if( ptr2 > 0 ):
                                # If Virtual host specification in config, find the path
                                ptr2 = ptr2 + 15
                                ptr1 = vhostfile.rfind( "<VirtualHost", 0, ptr2 )
                                vhosttag = vhostfile[ptr1:ptr2]
                                if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                                    vsearch = False
                                    vhdfound = True
                                    lprint( "Virtual Host Directive found." )
                                    lprint( vhosttag )
                                    ptr1 = vhosttag.find( "DocumentRoot" )
                                    if( ptr1 > 0 ):
                                        ptr2 = vhosttag[ptr1:].find( "\n" )
                                        gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].replace( '"', '' ).replace( "'", '' ).strip()
                                        break
                            else:
                                vsearch = False

            if not vhdfound:
                # Still have not found the path for lssdb. Look for fourth method.
                # This method is the old fashioned way of doing it and the virtual host directive can be in any file
                # The config file directive usually looks like:
                # # Include generic snippets of statements
                # Include conf.d/
                ptr2 = configFile.find( "config.d/" )
                if( ptr2 > 0 ):
                    ptr2 = ptr2 + 9
                    ptr1 = configFile.rfind( "Include", 0, ptr2 )
                    vhosttag = configFile[ptr1:ptr2]
                    lprint( "Possible virtual host definition found: %s" % vhosttag )
                    vhostfile = vhosttag.split( " " )[1]
                    os.chdir( apachedir + vhosttag )
                    cmd = 'dir /A:-D /B'
                    p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                    output = p.stdout.read()
                    filelist = output.split( "\n" )
                    for filename in filelist:
                        # Each file should contain something like the following:
                        # <VirtualHost *:80>
                        #    ServerName webtest190.com
                        #    ServerAlias *.webtest190.com
                        #    DocumentRoot /var/www/webtest
                        # </VirtualHost>
                        fh = open( filename, 'r' )
                        vhostfile = fh.read()
                        fh.close()
                        print vhostfile
                        vsearch = True
                        ptr2 = 0
                        while vsearch:
                            ptr2 = vhostfile.find( "\n</VirtualHost>", ptr2 )
                            if( ptr2 > 0 ):
                                # If Virtual host specification in config, find the path
                                ptr2 = ptr2 + 15
                                ptr1 = vhostfile.rfind( "<VirtualHost", 0, ptr2 )
                                vhosttag = vhostfile[ptr1:ptr2]
                                if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                                    vsearch = False
                                    lprint( "Virtual Host Directive found." )
                                    lprint( vhosttag )
                                    ptr1 = vhosttag.find( "DocumentRoot" )
                                    if( ptr1 > 0 ):
                                        ptr2 = vhosttag[ptr1:].find( "\n" )
                                        gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].replace( '"', '' ).replace( "'", '' ).strip()
                                        break
                            else:
                                vsearch = False

    """----------------------------------------------------------
    There is a difference on if you install in a system with an
    existing website or a virgin system. If this program just
    installed Apache then it's virgin.  If already installed
    then chances are it's being used and we want to create
    an extension to the base website URL.  For example if the base
    is www.ItsMyCompany.com then we want an lssdb.ItsMyCompany.com link.
    To do this we use the Apache Command apachectl -S (or on some systems
    httpctl -S) if the vhostname is localhost then we need to define
    a server name.  But then you need to query what address is going
    registered with the DNS boys.
    
    We can do both tasks with a single VHOST config file.  The file would
    look something like this:
    
    ----------------------------------
    ServerName www.ItsMyCompany.com
    DocumentRoot /var/www/somewhere
    
    <VirtualHost *:80>
        ServerName lssdb.ItsMyCompany.com
        ServerAlias *.ItsMyCompany.com
        DocumentRoot /var/www/lssdb
    </VirtualHost>
    ---------------------------------
    
    If the server name is already defined then you don't need the 
    first two lines.  The file is named same as ServerName in the
    file ie lssdb.ItsMyCompany.com and placed in the 
    sites-available folder.  It could alternately be placed
    in the sites-enabled folder and then the following command
    would not be needed but using sites-available follows current
    guidelines.  Then execute the command:
    
    a2ensite lssdb.ItsMyCompany.Com
    
    This makes a link in the sites-enabled folder to this file.
    Follow with the command:
    
    service apache2 reload
    
    And you are done.
    
    Be sure to set your host files on the machine you are browsing with
    To find the two URLs at the test machine IP address since no DNS
    Has the info yet.
    
    Debian (as TurnKey-Wheezy) uses Apache preset for virtual
    hosts.  The only difference is in the name of where the 
    virtual host config file sits.  So we find the apache2.conf 
    and look at all the includes until we find one we can recognize.
    
    Then we move to the folder /etc/apache2/sites-available and put in the
    canned file lssdb that contains the following directives:
    
    NameVirtualHost *:80
    NameVirtualHost *:443
    
    <VirtualHost *:80>
            ServerAdmin webmaster@lssdb.localhost
            DocumentRoot /var/www/lssdb/
    </VirtualHost>
    
    <VirtualHost *:443>
            SSLEngine on
            SSLCertificateFile /etc/ssl/certs/cert.pem
            ServerAdmin webmaster@lssdb.localhost
            DocumentRoot /var/www/lssdb/
    </VirtualHost>
    
    ScriptAlias /cgi-bin/ /var/www/cgi-bin/
    
    <Directory /var/www/lssdb>
            Options Indexes FollowSymLinks MultiViews
            Order allow,deny
            allow from all
    </Directory>
    
    Once the canned file is in place you make a link to it using the
    command "a2ensite lssdb" to link to sites-enabled.
    
    ------------------------------------------------------------"""
    # Sample file that is placed in the .conf folder for centos and debian
    prototypeConf = """<VirtualHost *:80>
    ServerName lssdb.webtest.com
    ServerAlias *.webtest.com
    DocumentRoot /var/www/lssdb
</VirtualHost>

<Directory "/var/www/lssdb">
    #
    # Possible values for the Options directive are "None", "All",
    # or any combination of:
    #   Indexes Includes FollowSymLinks SymLinksifOwnerMatch ExecCGI MultiViews
    #
    # Note that "MultiViews" must be named *explicitly* --- "Options All"
    # doesn't give it to you.
    #
    # The Options directive is both complicated and important.  Please see
    # http://httpd.apache.org/docs/2.4/mod/core.html#options
    # for more information.
    #
    Options Indexes FollowSymLinks MultiViews

    #
    # AllowOverride controls what directives may be placed in .htaccess files.
    # It can be "All", "None", or any combination of the keywords:
    #   Options FileInfo AuthConfig Limit
    #
    AllowOverride All

    #
    # Controls who can get stuff from this server.
    #
    Require all granted
</Directory>
"""
    # Sample file that is placed in sites-available folder for debian systems
    prototypeSitesAvailable = """<VirtualHost *:80>
    ServerName lssdb.webtest.com
    ServerAlias *.webtest.com
    DocumentRoot /var/www/lssdb
</VirtualHost>
<Directory /var/www/lssdb>
    Options Indexes FollowSymLinks
    AllowOverride All
    Require all granted
    Satisfy Any
</Directory>
"""

    if gv.sysPlatform == "Linux":
        confd = False
        vhdfound = False
        vsites = False
        override = False
        if( gv.distribution == "debian" ):
            ac.cprint( "Checking debian Linux Distribution for 'apache2.conf'" )
            cmd = "find / -name 'apache2.conf'"
        elif( gv.distribution == "Ubuntu" ):
            ac.cprint( "Checking Ubuntu Linux Distribution for 'apache2.conf'" )
            cmd = "echo %s | sudo -S find / -noleaf -type f -name 'apache2.conf'"
        else:
            ac.cprint ( "Checking Linux Distribution for 'httpd.conf'" )
            cmd = "find / -name 'httpd.conf'"
        lprint( "Looking for Apache configuration folder." )
        p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
        output = p.stdout.read()
        if ( len( output ) == 0 ):
            text = "The Apache Configuration file was not found."
            lprint( text, False )
            ac.cprint( text, ac.RED )
        else:
            result = output.split( "\n" )
            # Can get two results, the stored original or the users current copy
            for item in result:
                if ( ( gv.distribution == 'debian' ) or ( gv.distribution == 'Ubuntu' ) ):
                    if( item.find( ".conf" ) > 0 ):
                        filename = item
                else:
                    if ( item.find( "/conf" ) > 0 ):
                        filename = item
                        break
            lprint( "Found: %s" % filename )
            path = filename[0:filename.rfind( "/" ) + 1]
            if( ( gv.distribution == 'debian' ) or ( gv.distribution == 'Ubuntu' ) ):
                rootpath = path
                cfile = "apache2.conf"
            else:
                # Isolate the root path for Apache
                rootpath = path[0:( path[:-1].rfind( "/" ) ) + 1]
                cfile = "httpd.conf"
            os.chdir( path )
            fh = open( cfile , 'r' )
            configFile = fh.read()
            fh.close()
            lprint( configFile, False )
            # print configFile
            # Check for index.php
            lprint( "Checking for index.php reference in configuration file." )
            ptr1 = configFile.find( "<IfModule dir_module>" )
            if ptr1 > 0:
                ptr2 = configFile.find( "</IfModule>" )
                if configFile[ptr1:ptr2].find( "index.php" ) < 0:
                    text = "There is no reference for 'index.php' in the configuration file."
                    ac.cprint( text, ac.RED )
                    lprint( text, False )
                    ac.cprint( "Would you like to add index.php to the configuration file?", ac.WHITE, False )
                    showmethod = True
                    response = raw_input( "[yN]" )
                    if len( response ) > 0:
                        if response[0].lower() == "y":
                            try:
                                fh = open( cfile , 'w' )
                                fh.write( configFile.replace( "index.html", "index.html index.php" ) )
                                fh.close()
                                lprint( cfile + " was updated to include index.php" )
                            except:
                                text = cfile + " could not be updated to include index.php."
                                lprint( text, False )
                                ac.cprint( text, ac.RED )
                                ac.cprint( "Please update the file manually.", ac.YELLOW )
                        else:
                            lprint( "User has declined automated index.php update", False )
                            ac.cprint( "Please update the file manually.", ac.YELLOW )
                else:
                    lprint( "Reference to index.php found in configuration file." )
            else:
                lprint( "Error finding <IfModule dir_module> tag." )
            # Check for virtual hosts
            vsearch = True
            ptr2 = 0
            while vsearch:
                ptr2 = configFile.find( "</VirtualHost>", ptr2 )
                if( ptr2 > 0 ):
                    # If Virtual host specification in config, find the path
                    ptr2 = ptr2 + 15
                    ptr1 = configFile.rfind( "<VirtualHost", 0, ptr2 )
                    vhosttag = configFile[ptr1:ptr2]
                    if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                        vsearch = False
                        vhdfound = True
                        text = "Virtual Host Directive found."
                        ac.cprint( text, ac.GREEN )
                        lprint( text, False )
                        lprint( vhosttag )
                        ptr1 = vhosttag.find( "DocumentRoot" )
                        if( ptr1 > 0 ):
                            ptr2 = vhosttag[ptr1:].find( "\n" )
                            gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].replace( '"', '' ).replace( "'", '' ).strip()
                            text = "Document Root: %s" % gv.lssRoot
                            # Now look for an AllowOverride directive
                            override = findOverride( configFile, override )
                else:
                    vsearch = False
                    lprint( "Virtual Host tag not found inside configuration file", False )
            # At this point if lssRoot has length then we have a path
            # If not look for a directive using the -vhost.conf file
            if not vhdfound:
                # Still have not found the path for lssdb. Look for second method
                ptr2 = configFile.find( "-vhosts.conf" )
                if( ptr2 > 0 ):
                    ptr2 = ptr2 + 12
                    ptr1 = configFile.rfind( "Include", 0, ptr2 )
                    vhosttag = configFile[ptr1:ptr2]
                    lprint( "Virtual host definition found in -vhosts.conf: %s" % vhosttag )
                    vhostfile = vhosttag.split( " " )[1]
                    os.chdir( apachedir )
                    fh = open( vhostfile, 'r' )
                    vhostfile = fh.read()
                    fh.close()
                    vsearch = True
                    ptr2 = 0
                    while vsearch:
                        ptr2 = vhostfile.find( "\n</VirtualHost>", ptr2 )
                        if( ptr2 > 0 ):
                            # If Virtual host specification in config, find the path
                            ptr2 = ptr2 + 15
                            ptr1 = vhostfile.rfind( "<VirtualHost", 0, ptr2 )
                            vhosttag = vhostfile[ptr1:ptr2]
                            if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                                vsearch = False
                                vhdfound = True
                                lprint( "Virtual Host Directive found." )
                                ptr1 = vhosttag.find( "DocumentRoot" )
                                if( ptr1 > 0 ):
                                    ptr2 = vhosttag[ptr1:].find( "\n" )
                                    gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].replace( '"', '' ).replace( "'", '' ).strip()
                                # Now look for an AllowOverride directive
                                override = findOverride( configFile, override )
                        else:
                            vsearch = False
                            lprint( "Virtual Host tag not found -vhosts.conf file", False )
            # At this point if lssRoot has length then we have a path
            # If not look for a directive using the sites-enabled folder file
            # This is the latest recommended way to do vhosts and the file names
            # are usually the names of the URL like mywebsite.com.  The files should
            # really be in the sites-available folder with links to the sites-enabled folder.
            # The directive in the config file usually looks like:
            #     #Include the virtual host configurations:
            #     Include sites-enabled/
            if not vhdfound:
                # Still have not found the path for lssdb. Look for third method.
                vhosttag = "Include sites-enabled/"
                ptr1 = configFile.find( "\nInclude sites-enabled/" )
                if ptr1 < 0:
                    vhosttag = "IncludeOptional sites-enabled/"
                    ptr1 = configFile.find( "\nIncludeOptional sites-enabled/" )
                if( ptr1 >= 0 ):
                    vsites = True
                    ptr1 += 1
                    ptr2 = configFile[ptr1:].find( "\n" )
                    vhosttag = configFile[ptr1:ptr1 + ptr2]
                    lprint( "Virtual host definition found: %s" % vhosttag )
                    sitesEnabledPath = rootpath + 'sites-enabled'
                    sitesAvailablePath = rootpath + 'sites-available'
                    os.chdir( sitesEnabledPath )
                    lprint( "Searching %s for lssdb virtual host directives" % sitesEnabledPath )
                    cmd = 'ls'
                    p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                    output = p.stdout.read()
                    filelist = output.split( "\n" )
                    vsearch = False
                    for filename in filelist:
                        # Each file should contain something like the following:
                        # <VirtualHost *:80>
                        #    ServerName webtest190.com
                        #    ServerAlias *.webtest190.com
                        #    DocumentRoot /var/www/webtest
                        # </VirtualHost>
                        if ( filename <> "" ):
                            fh = open( filename, 'r' )
                            vhostfile = fh.read()
                            fh.close()
                            vsearch = True
                            ptr2 = 0
                            while vsearch:
                                ptr2 = vhostfile.find( "</VirtualHost>", ptr2 )
                                if( ptr2 > 0 ):
                                    # If Virtual host specification in config, find the path
                                    ptr2 = ptr2 + 15
                                    ptr1 = vhostfile.rfind( "<VirtualHost", 0, ptr2 )
                                    vhosttag = vhostfile[ptr1:ptr2]
                                    if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                                        vsearch = False
                                        vhdfound = True
                                        text = "Virtual Host Directive found."
                                        ac.cprint( text )
                                        lprint( text, False )
                                        text = vhosttag
                                        ac.cprint( text, ac.GREEN )
                                        lprint( text, False )
                                        ptr1 = vhosttag.find( "DocumentRoot" )
                                        if( ptr1 >= 0 ):
                                            ptr2 = vhosttag[ptr1:].find( "\n" )
                                            gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].strip()
                                            # Now look for an AllowOverride directive
                                            if not override:
                                                override = findOverride( vhostfile, override )
                                        break
                                    else:
                                        vsearch = False
                if not vhdfound:
                    text = "Virtual Host tag not found in sites-enabled."
                    lprint( text, False )
                    ac.cprint( text, ac.RED )

            if not vhdfound:
                # Still have not found the path for lssdb. Look for fourth method.
                # This method is the old fashioned way of doing it and the virtual host directive can be in any file
                # The config file directive usually looks like:
                # # Include generic snippets of statements
                # Include conf.d/
                ptr2 = configFile.find( "conf.d/" )
                confd = True
                if( ptr2 > 0 ):
                    ptr2 = ptr2 + 7
                    ptr1 = configFile.rfind( "Include", 0, ptr2 )
                    vhosttag = configFile[ptr1:ptr2]
                    lprint( "Possible virtual host definition found: %s" % vhosttag )
                    vhostpath = vhosttag.split( " " )[1]
                    vhostfile = vhostpath[vhostpath.rfind( "/" ) + 1:]
                    vhostpath = vhostpath[0:vhostpath.rfind( "/" )]
                    print "Changing to folder " + rootpath + vhostpath
                    includePath = rootpath + vhostpath
                    os.chdir( includePath )
                    lprint( "Searching %s for lssdb virtual host directives" % includePath )
                    cmd = 'ls ' + vhostfile
                    p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                    output = p.stdout.read()
                    filelist = output.split( "\n" )
                    for filename in filelist:
                        print "file: %s" % filename
                        # Each file should contain something like the following:
                        # <VirtualHost *:80>
                        #    ServerName webtest190.com
                        #    ServerAlias *.webtest190.com
                        #    DocumentRoot /var/www/webtest
                        # </VirtualHost>
                        if ( filename <> '' ):
                            fh = open( filename, 'r' )
                            vhostfile = fh.read()
                            fh.close()
                            vsearch = True
                            ptr2 = 0
                            while vsearch:
                                ptr2 = vhostfile.find( "\n</VirtualHost>", ptr2 )
                                if( ptr2 > 0 ):
                                    # If Virtual host specification in config, find the path
                                    ptr2 = ptr2 + 15
                                    ptr1 = vhostfile[0:ptr2].rfind( "<VirtualHost", 0, ptr2 )
                                    vhosttag = vhostfile[ptr1:ptr2]
                                    if( vhosttag.find( 'lssdb' ) > 0 or vhosttag.find( gv.lssRoot ) > 0 ):
                                        vsearch = False
                                        vhdfound = True
                                        print "Virtual Host Directive found."
                                        ac.cprint( vhosttag, ac.GREEN )
                                        lprint( vhosttag, False )
                                        ptr1 = vhosttag.find( "DocumentRoot" )
                                        if( ptr1 > 0 ):
                                            ptr2 = vhosttag[ptr1:].find( "\n" )
                                            gv.lssRoot = vhosttag[ptr1 + 13:ptr1 + ptr2].replace( '"', '' ).replace( "'", '' ).strip()
                                            # Now look for an AllowOverride directive
                                            if not override:
                                                override = findOverride( configFile, override )
                                            break
                                else:
                                    vsearch = False
                if not vhdfound:
                    text = "Virtual Host tag not found inside conf.d folder."
                    lprint( text, False )
                    ac.cprint( text, ac.RED )

        gv.VirtualHostSpecified = vsearch
        if vhdfound:
            text = "There is a specified location for the lssdb HTML files.\nThe Lssdb root: " + gv.lssRoot
            ac.cprint( text )
            lprint( text, False )
        else:
            text = "The lssdb document root is not defined. Using: " + gv.InstalledPath
            ac.cprint( text, ac.YELLOW )
            lprint( text, False )
            ac.cprint( "Please use this path when setting up the lssdb website.", ac.YELLOW )
            ac.cprint( "There was no virtual host directive on your system.  A prototype directive", ac.YELLOW )
            ac.cprint( "can be created for you to edit.", ac.YELLOW )
            ac.cprint( "Would you like to create the prototype directive?", ac.WHITE, False )
            showmethod = True
            response = raw_input( "[yN]" )
            if len( response ) > 0:
                if response[0].lower() == "y":
                    ecmd = ""
                    if gv.distribution == "debian":
                        rcmd = "service apache2 restart"
                        ecmd = "a2ensite"
                        filename = "lssdb.com"
                    elif gv.distribution == "Ubuntu":
                        rcmd = "sudo service apache2 restart"
                        ecmd = "sudo a2ensite"
                    elif gv.distribution == "fedora":
                        rcmd = "systemctl restart httpd.service"
                    else:
                        rcmd = "systemctl restart httpd.service"
                    if gv.distribution == "Ubuntu":
                        filename = "lssdb.conf"
                    if vsites:
                        fullpath = sitesAvailablePath
                        filename = "lssdb.conf"
                        cmd = "echo %s | sudo -S chmod 777 %s" % ( gv.suPassword, fullpath )
                        p = subprocess.Popen( cmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                        output = p.stdout.read()
                        p.wait()
                        filetext = prototypeSitesAvailable
                        endtext = "You must edit the file to match your site needs.\n"
                        endtext += "Then enable the site with the following command:\n"
                        endtext += "\t'%s %s\n" % ( ecmd, filename )
                        endtext += "and restart apache with the following command:\n"
                        endtext += "\t%s" % rcmd
                    else:
                        fullpath = includePath
                        filename = 'lssdb.conf'
                        filetext = prototypeConf
                        endtext = "You must edit the file to match your site needs\n"
                        endtext += "and the use the command %s to\nactivate the web site." % rcmd

                    text = "The file %s is being written to the %s folder." % ( filename, fullpath )
                    lprint( text, False )
                    ac.cprint( text )
                    try:
                        writefilehandle = open( fullpath + "/" + filename , "w" )
                        writefilehandle.write( filetext )
                        writefilehandle.close()
                        lprint( endtext, False )
                        ac.cprint( endtext, ac.YELLOW )
                        showmethod = False
                        # Do a final restart to make any changes work
                        if gv.distribution == "debian":
                            rcmd = "service apache2 restart"
                        elif gv.distribution == "Ubuntu":
                            rcmd = "sudo service apache2 restart"
                        elif gv.distribution == "fedora":
                            rcmd = "systemctl restart httpd.service"
                        else:
                            rcmd = "systemctl restart httpd.service"
                        p = subprocess.Popen( rcmd, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.STDOUT )
                        status = p.stdout.read()
                        p.wait()
                        override = True
                    except:
                        text = "There was a problem writing the prototype virtual host directive.\n"
                        text += "You must create the file manually following instructions below."
                        lprint( text, False )
                        ac.cprint( text, ac.RED )
            if showmethod:
                lprint( "You have two modern methods of creating a virtual host file." )
                lprint( "For both methods you need to create a file with structure\nsimilar to the following:" )
                lprint( """<VirtualHost *:80>\n\tServerName lssdb\n\tServerAlias *.yourexistingwebsite.com\n\tDocumentRoot /var/www/lssdb\n</VirtualHost>""" )
                lprint( "On debian systems with at least version 2.4 Apache you should label" )
                lprint( "this file lssdb and place it in the folder:" )
                lprint( "\t/etc/apache2/sites-available/" )
                lprint( "Then run: a2ensite lssdb to construct a link to the sites-enabled folder." )
                lprint( "\nIf you have a system with httpd instead of apache2 then label the file lssdb.conf and place it in the folder:" )
                lprint( "\t/etc/httpd/conf.d/" )
                if( not confd ):
                    lprint( "You should then insure that the statement: Include conf.d   is in the httpd.conf file." )
            if not override:
                text = "No AllowOverride directive was found for the document directory.\n"
                text += "This may cause problems and should be corrected.  Please add:\n"
                text += "'AllowOverride All'  to your LssDB <Document> tag in your configuration file."
                lprint( text, False )
                ac.cprint( text, ac.RED )


def main():
    gv.init()
    print "Performing Server Operations in stand alone mode."
    try:
        gv.LogFileHandle = open( "lssdb_log.txt", "w" )
    except:
        print "Unable to open log file. Continuing without."
    try:
        ServerOps()
    except KeyboardInterrupt:
        gv.LogFileHandle.close()
        exit()
    gv.LogFileHandle.close()
    print "Program completed normally."

if __name__ == '__main__':
    try:
        main()
    except:
        raise



